<?php

class Setup_SNMP_Hook_Helper {

	private static function api_call ($method, $url, array $data, $user, $password) {

		$handle = curl_init($url);

		curl_setopt($handle, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($handle, CURLOPT_HTTPHEADER, array('content-type: application/json'));
		curl_setopt($handle, CURLOPT_CUSTOMREQUEST, $method);
		curl_setopt($handle, CURLOPT_USERPWD, "{$user}:{$password}");
		curl_setopt(
			$handle,
			CURLOPT_POSTFIELDS,
			json_encode($data)
		);

		$result = curl_exec($handle);
		curl_close($handle);
		return $result;

	}

	public static function create_snmp_user ($snmp_user, $authpass, $privpass) {

		$log = op5Log::instance('ninja');

		/**
		 * We had to choose a passphrase length and decided on 512 bits, so for
		 * now it is enforced to 512 bits.
		 */
		if (strlen($authpass) != 32 || strlen($privpass) != 32) {
			throw new Exception("Requires at least 512 bits for authpass and privpass");
		}

		$output = array();
		$status = 0;
		$created_user = true;

		exec('sudo /usr/bin/systemctl stop snmpd', $output, $status);
		if ($status != 0) {
			$log->log('error', "Could not stop snmpd before user creation: " . implode(", ", $output));
			return false;
		}

		exec("sudo /usr/bin/net-snmp-create-v3-user -ro -a SHA -A $authpass -x AES -X $privpass $snmp_user", $output, $status);
		if ($status != 0) {
			$log->log('error', "Failed to execute 'net-snmp-create-v3-user': " . implode(", ", $output));
			$created_user = false;
		} else {
			$log->log('notice', "Added snmpv3 user '$snmp_user'");
		}

		exec('sudo /usr/bin/systemctl start snmpd', $output, $status);
		if ($status != 0) {
			$log->log('error', "Could not start snmpd after user creation: " . implode(", ", $output));
		}

		/**
		 * If we failed to add an snmp user we should not apply the management pack
		 * that requires the user to function, we instead leave it up to the user
		 * to handle this problem.
		 */
		return $created_user;

	}

	public static function add_self_monitoring ($host, $user, $password, $snmp_user, $authpass, $privpass) {

		$log = op5Log::instance('ninja');
		$result = self::api_call("PATCH", "http://localhost/api/config/host/monitor", array(
			'host_name' => $host,
			'hostgroups' => array(
				'network',
				'unix-servers',
				'op5_monitor_servers',
				'OP5 Monitor Server'
			),
			'_OP5SECRET__AES_KEY' => $privpass,
			'_OP5SECRET__SHA_KEY' => $authpass,
			'_OP5SECRET__USERNAME' => $snmp_user
		), $user, $password);

		if ($result === false) {
			$log->log('error', "Failed to add self-monitoring to host 'monitor' -> " . $host);
			return false;
		}

		$result = self::api_call("POST", "http://localhost/api/config/change", array(), $user, $password);

		if ($result === false) {
			$log->log('error', "Failed to commit self-monitoring to host 'monitor'");
			return false;
		}

		$log->log('notice', "Added self-monitoring to host 'monitor'");
		return true;

	}

}

Event::add('ninja.setup.snmp', function () {

	$log = op5Log::instance('ninja');
	$log->log('notice', "Updating OP5 Monitor with an SNMP user and self-monitoring");

	$data = Event::$data;
	$user = $data['user']['username'];
	$password = $data['user']['password'];
	// See README.md for reasoning around op5-snmp-user
	$snmp_user = "op5-snmp-user";

	// Create SNMPv3 user
	// Generate pseudo random 32 character passwords
	$authpass = bin2hex(openssl_random_pseudo_bytes(16));
	$privpass = bin2hex(openssl_random_pseudo_bytes(16));

	if (Setup_SNMP_Hook_Helper::create_snmp_user($snmp_user, $authpass, $privpass)) {
		$hostname = strtolower(gethostname());

		/* Check names that are likely to be duplicate across peers and add a UUID */
		if (strpos($hostname,"localhost") !== FALSE) {
			$hostname = "monitor-localhost-" . uniqid();
		} else if ($hostname === "op5-system") {
			$hostname = "monitor-op5system-" . uniqid();
		} else {
			$hostname = "monitor-" . $hostname;
		}

		if (
			Setup_SNMP_Hook_Helper::add_self_monitoring(
				$hostname, $user, $password, $snmp_user, $authpass, $privpass
			)
		) {
			$log->log('notice', "Successfully prepared OP5 Monitor with an SNMP user and self-monitoring");
		}
	}

});
